import { merge, NEVER, of, ReplaySubject, Subject } from 'rxjs';
import { distinctUntilChanged, map, multicast, refCount, scan } from 'rxjs/operators';
import { Lens } from './lens';
/**
 * A stateful container that exposing a stream of state values that users can subscribe to.
 *
 * A Telescope is an endo-profunctor for a type U which means it accepts new U values and
 * produces U values on the associated stream.
 */
var Telescope = /** @class */ (function () {
    function Telescope(evolver, stream) {
        this.evolver = evolver;
        this.stream = stream.pipe(distinctUntilChanged());
    }
    Telescope.of = function (initialState) {
        var evolutions = new Subject();
        var evolver = function (evolution) { return evolutions.next(evolution); };
        var stream = merge(of(function (u) { return u; }), evolutions, NEVER)
            .pipe(scan(function (acc, elem) { return elem(acc); }, initialState), 
        // LOCAL MOD: compatible with https://github.com/ReactiveX/rxjs/pull/4816
        multicast(function () { return new ReplaySubject(1); }), refCount());
        return new Telescope(evolver, stream);
    };
    Telescope.prototype.dimap = function (to, from) {
        return this.magnify(new Lens(to, function (p, u) { return from(p); }));
    };
    Telescope.prototype.uplift = function (value) {
        var _this = this;
        var lastC = value;
        var to = function (u) { return ({ first: u, second: lastC }); };
        var constrain = function (evolution) { return function (u) {
            var newPair = evolution({ first: u, second: lastC });
            lastC = newPair.second;
            return newPair.first;
        }; };
        return new Telescope(function (evolution) { return _this.evolver(constrain(evolution)); }, this.stream.pipe(map(to), distinctUntilChanged()));
    };
    Telescope.prototype.evolve = function (evolution) {
        this.evolver(evolution);
    };
    Telescope.prototype.update = function (newState) {
        this.evolve(function (_) { return newState; });
    };
    Telescope.prototype.magnify = function (lens) {
        var _this = this;
        var extend = function (evolution) { return function (u) { return lens.setter(evolution(lens.getter(u)), u); }; };
        return new Telescope(function (evolution) { return _this.evolver(extend(evolution)); }, this.stream.pipe(map(lens.getter), distinctUntilChanged()));
    };
    return Telescope;
}());
export { Telescope };
//# sourceMappingURL=telescope.js.map