"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var lens_1 = require("./lens");
/**
 * A stateful container that exposing a stream of state values that users can subscribe to.
 *
 * A Telescope is an endo-profunctor for a type U which means it accepts new U values and
 * produces U values on the associated stream.
 */
var Telescope = /** @class */ (function () {
    function Telescope(evolver, stream) {
        this.evolver = evolver;
        this.stream = stream.pipe(operators_1.distinctUntilChanged());
    }
    Telescope.of = function (initialState) {
        var evolutions = new rxjs_1.Subject();
        var evolver = function (evolution) { return evolutions.next(evolution); };
        var stream = rxjs_1.merge(rxjs_1.of(function (u) { return u; }), evolutions, rxjs_1.NEVER)
            .pipe(operators_1.scan(function (acc, elem) { return elem(acc); }, initialState), 
        // LOCAL MOD: compatible with https://github.com/ReactiveX/rxjs/pull/4816
        operators_1.multicast(function () { return new rxjs_1.ReplaySubject(1); }), operators_1.refCount());
        return new Telescope(evolver, stream);
    };
    Telescope.prototype.dimap = function (to, from) {
        return this.magnify(new lens_1.Lens(to, function (p, u) { return from(p); }));
    };
    Telescope.prototype.uplift = function (value) {
        var _this = this;
        var lastC = value;
        var to = function (u) { return ({ first: u, second: lastC }); };
        var constrain = function (evolution) { return function (u) {
            var newPair = evolution({ first: u, second: lastC });
            lastC = newPair.second;
            return newPair.first;
        }; };
        return new Telescope(function (evolution) { return _this.evolver(constrain(evolution)); }, this.stream.pipe(operators_1.map(to), operators_1.distinctUntilChanged()));
    };
    Telescope.prototype.evolve = function (evolution) {
        this.evolver(evolution);
    };
    Telescope.prototype.update = function (newState) {
        this.evolve(function (_) { return newState; });
    };
    Telescope.prototype.magnify = function (lens) {
        var _this = this;
        var extend = function (evolution) { return function (u) { return lens.setter(evolution(lens.getter(u)), u); }; };
        return new Telescope(function (evolution) { return _this.evolver(extend(evolution)); }, this.stream.pipe(operators_1.map(lens.getter), operators_1.distinctUntilChanged()));
    };
    return Telescope;
}());
exports.Telescope = Telescope;
//# sourceMappingURL=telescope.js.map